<link
  rel="stylesheet"
  href="{{ url_for('static', filename='css/drawflow.min.css') }}"
/>
<link
  rel="stylesheet"
  href="{{ url_for('static', filename='css/micromodal.css') }}"
/>
<link
  rel="stylesheet"
  type="text/css"
  href="{{ url_for('static', filename='css/beautiful.css') }}"
/>
<link
  rel="stylesheet"
  href="https://cdnjs.cloudflare.com/ajax/libs/font-awesome/5.13.0/css/all.min.css"
  integrity="sha256-h20CPZ0QyXlBuAw7A+KluUYx/3pK+c7lYEpqLTlxjYQ="
  crossorigin="anonymous"
/>
<link
  href="https://fonts.googleapis.com/css2?family=Roboto&display=swap"
  rel="stylesheet"
/>
<div id="node_styles">
  {% if node_styles|length %}
  <style>
    {{ node_styles|safe }}
  </style>
  {% endif %}
</div>
