{% extends "base.html" %} {% block content %}
<link
  rel="stylesheet"
  href="{{ url_for('static', filename='css/index.css') }}"
/>
<body>
  <div class="container-fluid">
    <div class="row home">
      <div class="col-md-12 splash">
        <a>
          <div class="intro">
            <h1>SST Hierarchical Composition Tool</h1>
          </div>
          <button
            onclick="location.href='/demo'"
            type="button"
            class="btn btn-default CTA"
          >
            Demo
          </button>
          <button
            onclick="location.href='/canvas'"
            type="button"
            class="btn btn-default CTA"
          >
            Create a new diagram
          </button>

          <form method="POST" action="/import" enctype="multipart/form-data">
            <div class="upload-btn-wrapper">
              <button type="button" class="btn btn-default CTA">
                Load diagram
              </button>
              <input
                type="file"
                name="saved_config_file"
                onchange="this.form.submit();"
              />
            </div>
          </form>

          <div class="sub">
            <p>
              Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do
              eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut
              enim ad minim veniam, quis nostrud exercitation ullamco laboris
              nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in
              reprehenderit in voluptate velit esse cillum dolore eu fugiat
              nulla pariatur. Excepteur sint occaecat cupidatat non proident,
              sunt in culpa qui officia deserunt mollit anim id est laborum.
            </p>
          </div>
        </a>
      </div>
    </div>
  </div>
</body>
{% endblock %}
