<!DOCTYPE html>
<html lang="en">
  <head>
    <meta charset="UTF-8" />
    <meta name="viewport" content="width=device-width, initial-scale=1.0" />
    <meta http-equiv="X-UA-Compatible" content="ie=edge" />
    <title>SST Hierarchical Composition Tool</title>
    <meta name="description" content="SST Hierarchical Composition Tool" />
  </head>
  {% block content %}{% endblock %} {% include "scripts.html" %}
</html>
